;-----------
OneCharDown
	LD A,L
	ADD A,32
	LD L,A
	RET NC				;stay in the same screen segment
	
	LD A,H				;next screen segment
	ADD A,8
	LD H,A
	RET
	
;-------------------
DrawSprite
	;HL - screen position
	;DE - address
	;BC -size (width,height)
	
DrawSprite2	
	PUSH HL
	PUSH BC
	
DrawSprite3
	;one row of characters			
	
	PUSH BC
	LD B,8
	LD C,H
	
DrawSprite4		
	;one character
	LD A,(DE)
	LD (HL),A
	INC H				;next line on screen
	INC DE				;next data byte
	DJNZ DrawSprite4
	
	LD H,C				;restore top line of characters
	INC L				;next character right
	POP BC
	DJNZ DrawSprite3
	
	POP BC
	POP HL				;go to leftmost character
	CALL OneCharDown	;next row
	DEC C
	JR NZ,DrawSprite2
	RET
	
;--------------------------	
AttributeAddress
	LD A,80						;graphics address to attribute address
	SRL H
	SRL H
	SRL H
	ADD A,H
	LD H,A
	RET
	
;--------------------------
DrawBorder
;draw an empty box with a border (for text or picture to be placed later)
	;HL: position of left upper box corner
	;BC: size,B=width,C=height
	
	LD HL,(BorderCorner) 
	LD A,(BoardSize)
	SLA A
	INC A
	INC A
	LD B,A
	LD C,A

DrawBorder1		
	PUSH HL
	PUSH BC
	CALL ClearArea
	POP BC
	POP HL
	
	PUSH HL
	PUSH BC
	LD A,70
	CALL PaintArea
	POP BC
	POP HL
	

DrawBorder2
;you can jump here if you don't want to clear the inside of the box
	
;-------
DrawBorderA
;draw top edge	
	PUSH BC
	PUSH HL
	LD DE,BorderTL					;left top corner
	CALL DrawBorderSprite
	POP HL
	POP BC
	
	INC L							;horizontal line width= border width - 2 (for corners)
	DEC B
	DEC B
;top line
	PUSH BC							;store size
	LD DE,BorderTHor	
DrawBorderA2
	PUSH HL
	PUSH DE
	PUSH BC
	CALL DrawBorderSprite
	POP BC
	POP DE
	POP HL
	INC L
	DJNZ DrawBorderA2
	
	LD DE,BorderTR
	PUSH HL
	CALL DrawBorderSprite			;right top corner
	POP HL
	POP BC							;restore size

	
;-------										
;draw left and right edge
DrawBorderB
	PUSH BC							;store size
	DEC C
	DEC C
	
DrawBorderB2
	CALL OneCharDown	

	LD DE,BorderL
	PUSH HL
	PUSH BC							;draw right edge
	CALL DrawBorderSprite
	POP BC
	POP HL
	
	PUSH HL							;store right edge
	
	LD A,L							;go to left edge
	SUB B
	DEC A
	LD L,A							
	
	LD DE,BorderR
	PUSH BC							;draw left edge
	CALL DrawBorderSprite	
	POP BC
	
	POP HL							;restore right edge
	
	DEC C							;do it C times
	JR NZ,DrawBorderB2

	POP BC							;restore size
							
;draw bottom edge
DrawBorderC
	CALL OneCharDown
	
	LD DE,BorderBR
	PUSH HL
	PUSH BC							;bottom right corner
	CALL DrawBorderSprite	
	POP BC
	POP HL

	DEC L							
;bottom line
	LD DE,BorderBHor	
DrawBorderC2
	PUSH HL
	PUSH DE
	PUSH BC
	CALL DrawBorderSprite
	POP BC
	POP DE
	POP HL
	DEC L
	DJNZ DrawBorderC2
	
	LD DE,BorderBL					;bottom left corner
	CALL DrawBorderSprite
	RET

;----	
DrawBorderSprite
	LD BC,256*1+1
	CALL DrawSprite
	RET	
	
;---------------
;border graphics	

BorderTL	.BYTE	  0,127,127, 96,111,104,104,104
BorderTHor	.BYTE	  0,255,255,  0,255,  0,  0,  0
BorderTR	.BYTE	  0,254,254,  6,246, 22, 22, 22
BorderR		.BYTE	  104,104,104,104,104,104,104,104
BorderL		.BYTE	  22, 22, 22, 22, 22, 22, 22, 22
BorderBL	.BYTE	  104,104,104,111, 96,127,127,  0
BorderBHor	.BYTE	  0,  0,  0,255,  0,255,255,  0
BorderBR	.BYTE	  22, 22, 22,246,  6,254,254,  0




;----------------------
ClearArea
;HL - position,BC - size


	PUSH BC
	POP DE	
	
ClearArea2
	PUSH DE
	LD E,L
	
ClearArea3	
	LD B,8
	XOR A
	LD C,H
ClearArea4					;clear one char
	LD (HL),A	
	INC H
	
	DJNZ ClearArea4
	LD H,C
	
	INC L
	DEC D
	JR NZ,ClearArea3	
	
	LD L,E					;back to start of the line
	CALL OneCharDown
	POP DE					;restore E-width
	DEC E
	JR NZ,ClearArea2
	RET


;----------------------------
ClearScreen
	XOR A
	OUT (254),A
	LD (23624),A 
	LD HL,16384
	LD DE,16385
	LD BC,6144
	LD (HL),A
	LDIR
	
	LD A,66
	LD HL,22528
	LD DE,22529
	LD BC,768
	LD (HL),A
	LDIR
	RET
	
;-----------------------------
PaintArea
;HL-position,BC-size,A-atribute
	
	PUSH AF
	CALL AttributeAddress
	POP AF
	
PaintArea2	
	LD D,B
	LD E,L
PaintArea3	
	LD (HL),A					;write attribute
	INC L						;next char
	DJNZ PaintArea3	
	
	LD L,E						;restore line beginning
	LD B,D						;restore width
	
	LD DE,32
	ADD HL,DE					;one line down

	DEC C
	JR NZ,PaintArea2		
	RET	
;----------------------------
DrawCursor
	
	CALL CursorXYtoHL
	
DrawCursor2
	LD DE,CursorGraph
	CALL DrawCursor_Char
	INC L
	CALL DrawCursor_Char
	DEC L
	CALL OneCharDown
	CALL DrawCursor_Char
	INC L
	CALL DrawCursor_Char
	RET
;-------	
DrawCursor_Char	
	LD B,8
	LD C,H
DrawCursor_Char2	
	LD A,(DE)
	
DrawCursor_SMD		
	OR (HL)				;self modyfing code XOR (HL) for clearing, OR (HL) for drawing
	LD (HL),A
	INC DE
	INC H
	DJNZ DrawCursor_Char2
	LD H,C
	RET	
	
;---------------------------	
MarkSelectedJewel
	LD A,(SelectedJewel)
	OR A
	RET Z					;no jewel selected
	
;drawing border around jewel	
	CALL SelectedJewelXYtoHL
	
	CALL DrawCursor2	
	RET
	
	
;--------------------------	
BoardXYtoHL
;B -X pos,C - Y pos

	LD HL,(BoardPos)
;X pos
	LD A,B
	OR A					;left edge, no need for changes?
	JR Z,BoardXYtoHL3		

BoardXYtoHL2
	INC L
	INC L		
	DJNZ BoardXYtoHL2	
	
;Y pos	
BoardXYtoHL3
	LD A,C			;top edge, no need for changes?
	OR A
	RET Z
	
BoardXYtoHL4
	CALL OneCharDown
	CALL OneCharDown
	DEC C
	JR NZ,BoardXYtoHL4
	RET
	
;-------------
CursorXYtoHL
	LD A,(CursorX)
	LD B,A
	LD A,(CursorY)
	LD C,A
	CALL BoardXYtoHL
	LD (CursorHL),HL
	RET
	
;-------------
SelectedJewelXYtoHL
	LD A,(SelectedX)
	LD B,A
	LD A,(SelectedY)
	LD C,A
	CALL BoardXYtoHL
	RET
	
	
;-------------	
CursorGraph
	.BYTE	127,128,128,128,128,128,128,128
	.BYTE	254,  1,  1,  1,  1,  1,  1,  1
	.BYTE	128,128,128,128,128,128,128,127
	.BYTE	  1,  1,  1,  1,  1,  1,  1,254	
	
;----------------------------
CopyBackground
	LD A,(BackgroundColor)

;paint attribute
	LD HL,22528
	LD DE,22529
	LD BC,767
	LD (HL),A
	LDIR

;copy graphics
	LD A,192
	LD HL,BackgroundPic	
	LD DE,16384
	
CopyBackground2
	REPT 32
	LDI
	ENDR
	DEC A
	JR NZ,CopyBackground2
	RET
	
BackgroundPic	
	INCBIN <Background.bin>
	
	
;---------------------	
;jewels

DrawJewel
;HL - position
;DE - jewel address

	LD BC,2*256+2
	PUSH HL
	CALL DrawSprite
	POP HL
	
	LD A,(DE)				;attribute
	LD BC,2*256+2
	CALL PaintArea
	RET


Jewel1
	.BYTE	  0,  0,  7,  8, 19, 39, 46, 47
	.BYTE	  0,  0,192, 32,144, 72,136,  8
	.BYTE	 46, 21, 10,  5,  2,  1,  0,  0
	.BYTE	136, 16, 32, 64,128,  0,  0,  0
	.BYTE 65
	
Jewel2
	.BYTE	  0,  0,  1,  1,  3,  2,  5,  6
	.BYTE	  0,  0,  0,  0,128,128, 64, 64
	.BYTE	 13, 10, 28, 26, 52, 63,  0,  0
	.BYTE	 32, 32, 16, 16,  8,248,  0,  0
	.BYTE	  2
	
Jewel3
	.BYTE	  0,  0,  3, 28, 51, 47, 46, 45
	.BYTE	  0,  0,192, 56,204,228,196,132
	.BYTE	 43, 46, 44, 48, 28,  3,  0,  0
	.BYTE	 36, 68,132, 12, 56,192,  0,  0
	.BYTE	  3	
	
Jewel4
	.BYTE	  0,  0,  1,  2,  5, 11, 23, 42
	.BYTE	  0,  0,  0,128, 64,160,208,136
	.BYTE	 21, 10,  4,  2,  1,  0,  0,  0
	.BYTE	 16, 32, 64,128,  0,  0,  0,  0
	.BYTE	  4
	
Jewel5
	.BYTE	  0,  0,  1,  1,  2,  5, 11, 55
	.BYTE	  0,  0,  0,  0,128, 64,160, 24
	.BYTE	 10,  4,  2,  1,  1,  0,  0,  0
	.BYTE	 32, 64,128,  0,  0,  0,  0,  0
	.BYTE	 69	
	
Jewel6
	.BYTE	  0,  0,  0, 15, 23, 27, 28, 29
	.BYTE	  0,  0,  0,240,232,200,  8,136
	.BYTE	 29, 28, 24, 16, 15,  0,  0,  0
	.BYTE	  8, 72, 40, 24,240,  0,  0,  0
	.BYTE	  6
	
Jewel7
	.BYTE	  0,  0,  7,  8, 22, 45, 46, 45
	.BYTE	  0,  0,192, 32, 16,  8,136,  8
	.BYTE	 46, 45, 22,  8,  7,  0,  0,  0
	.BYTE	136,  8, 16, 32,192,  0,  0,  0
	.BYTE	 71	
	
Jewel8
	.BYTE	  0,  0, 15, 16, 39, 45, 43, 46
	.BYTE	  0,  0,224, 16,200,168, 72,136
	.BYTE	 45, 42, 36, 16, 15,  0,  0,  0
	.BYTE	 72,136,  8, 16,224,  0,  0,  0
	.BYTE	  66
	
Jewel9
	.BYTE	  0,  0, 31, 32, 47, 47, 32, 47
	.BYTE	  0,  0,248,  4,244,228,  4,196
	.BYTE	 32, 47, 32, 44, 32, 31,  0,  0
	.BYTE	  4,  4,  4,  4,  4,248,  0,  0
	.BYTE	  68
	
Jewel10
	.BYTE	  0,  0,  7,  8, 23, 45, 42, 45
	.BYTE	  0,  0,192, 32,144, 72,200,200
	.BYTE	 47, 38, 16,  8,  7,  0,  0,  0
	.BYTE	136,  8, 16, 32,192,  0,  0,  0
	.BYTE	  69
	
Jewel11	
	.BYTE	  0,  0,  1,  3,  7, 15, 30, 29
	.BYTE	  0,  0,  0,128,192, 96,176, 16
	.BYTE	 26, 20,  9,  4,  2,  1,  0,  0
	.BYTE	 16,144, 32, 64,128,  0,  0,  0
	.BYTE	 70
	
	
;-------------------------
Explosion1	
	.BYTE	  0,  0,  0,  0,  0,  1,  3,  7
	.BYTE	  0,  0,  0,  0,  0,192,224,240
	.BYTE	  7,  7,  3,  1,  0,  0,  0,  0
	.BYTE	240,240,224,192,  0,  0,  0,  0
	
Explosion2
	.BYTE	  0,  0,  0,  1,  7, 15, 15, 31
	.BYTE	  0,  0,  0,192,240,248,248,252
	.BYTE	 31, 31, 15, 15,  7,  1,  0,  0
	.BYTE	252,252,248,248,240,192,  0,  0
	
Explosion3
	.BYTE	  0,  0,  1,  7, 15, 31, 31, 62
	.BYTE	  0,  0,192,240,248,252,124,190
	.BYTE	 63, 62, 31, 31, 15,  7,  1,  0
	.BYTE	126,190,124,252,248,240,192,  0
	
Explosion4
	.BYTE	  0,  0,  1,  7, 15, 30, 29, 58
	.BYTE	  0,  0,192,240,248,188, 92,174
	.BYTE	 61, 58, 29, 30, 15,  7,  1,  0
	.BYTE	 94,174, 92,188,248,240,192,  0
	
	
Explosion5
	.BYTE	  0,  0,  1,  6, 13, 26, 21, 42
	.BYTE	  0,  0,192,176, 88,172, 84, 42
	.BYTE	 52, 42, 21, 26, 13,  6,  1,  0
	.BYTE	 22, 42, 84,172, 88,176,192,  0
	
Explosion6
	.BYTE	  0,  0,  1, 10, 20,  8, 16, 40
	.BYTE	  0,  0, 64,168, 20,  8,  4, 10
	.BYTE	 16, 40, 16,  8, 20, 10,  1,  0
	.BYTE	  4, 10,  4,  8, 20,168, 64,  0

Explosion7
	.BYTE	  0,  0,  1,  4,  0, 16,  0, 32
	.BYTE	  0,  0, 64, 16,  4,  0,  0,  2
	.BYTE	  0, 32,  0,  0,  0,  8,  1,  0
	.BYTE	  0,  2,  0,  0,  4,  8, 64,  0
	
	
Explosion8
	REPT 32
	.BYTE 0
	ENDR
	

EndingPic
	.BYTE	  0,  0,  0,  0,  0,  0,  0,  0
	.BYTE	  0,  0,  0,  0,  0,  0,  0,  0
	.BYTE	  8, 28, 28, 62, 54, 54, 99, 99
	.BYTE	  0,  0,  0,  0,  0,  0,  0,  0
	.BYTE	  0,  0,  0,  0,  0,  0,  0,  0
	.BYTE	  0,  0,  0,  0,  0,127, 63, 28
	.BYTE	  0,  0,  0,  0,  1,249,249,  3
	.BYTE	 99,193,193,193,128,159,159,  0
	.BYTE	  0,128,128,128,  0,255,255,  0
	.BYTE	  0,  0,  0,  0,  0,255,254, 28
	.BYTE	 14,  7,  3,  1,  0,  0,  0,  0
	.BYTE	  3,  3,134,198,226,112, 56, 28
	.BYTE	  0,  0,  0,  0,  0,  0,  0,  0
	.BYTE	  0, 48, 48, 49, 25, 25, 24, 12
	.BYTE	 56,112,224,192,128,  0,  0,  0
	.BYTE	  0,  0,  0,  0,  0,  0,  0,  0
	.BYTE	 14,  7, 35, 49, 48, 96, 96, 96
	.BYTE	  0,  0,128,193,227, 71, 14, 28
	.BYTE	 12, 76,198,198,134,  3,  3,  3
	.BYTE	  0,  0,  0,  0,  0,  0,  0,  0
	.BYTE	  0,  0,  0,  1,  1,  1,  3,  3
	.BYTE	192,192,192,129,131,135, 14, 28
	.BYTE	 56,113,227,193,128,  0,  0,  0
	.BYTE	  1,  1,129,192,224,112, 56, 28
	.BYTE	128,128,128,192,192,192, 96, 96
	.BYTE	  3,  6,  6,  7,  7, 15, 14, 12
	.BYTE	 56,112,224,192,128,  0,  0,  0
	.BYTE	  0,  0,  0,  0,  0,  0,  0,  0
	.BYTE	 14,  7,  3,  1,  0,  0,  0,  0
	.BYTE	 96, 48,176,240,240,120, 56, 24
	
JewelsPassword
	.BYTE	  7,  8, 23, 45, 90,181,187,183
	.BYTE	192, 32,208,104,228,242,242,242
	.BYTE	191,159, 71, 32, 16,  8,  7,  0
	.BYTE	226,194,  4,  8, 16, 32,192,  0
	.BYTE 5
	.BYTE	  7, 24, 39, 79, 95,191,191,191
	.BYTE	192, 48,136,196,164,210,226,210
	.BYTE	191,191, 95, 79, 39, 24,  7,  0
	.BYTE	226,210,164,196,136, 48,192,  0
	.BYTE 71
	.BYTE	  3, 12, 51, 79, 95, 95, 94, 93
	.BYTE	192, 48,204,242,242,226,194,146
	.BYTE	 91, 86, 92, 72, 48, 12,  3,  0
	.BYTE	 34, 66,130,  2, 12, 48,192,  0
	.BYTE 3
	.BYTE	  0, 63, 64, 95, 95, 95, 64, 95
	.BYTE	  0,252,  2,250,242,226,  2,194
	.BYTE	 95, 64, 94, 64, 88, 64, 63,  0
	.BYTE	130,  2,  2,  2,  2,  2,252,  0
	.BYTE 4
	.BYTE	 31, 32, 95, 95, 93, 91, 94, 93
	.BYTE	252,  2,253,249,245,121,241,233
	.BYTE	 91, 95, 93, 90, 80, 32, 31,  0
	.BYTE	209,161, 65,129,  1,  2,252,  0
	.BYTE 66
	.BYTE	  0,  1,  2,  5, 11, 23, 47, 94
	.BYTE	128, 64,160,208,232,244,106,209
	.BYTE	 45, 23, 10,  5,  2,  1,  0,  0
	.BYTE	162, 68,136, 16, 32, 64,128,  0
	.BYTE 68
	.BYTE	127, 64, 79, 87, 91, 92, 93, 93
	.BYTE	255,  1,249,241,225,  1,193,145
	.BYTE	 93, 92, 89, 80, 64, 64,127,  0
	.BYTE	 17, 17,241,  9,  5,  1,255,  0
	.BYTE 6
	.BYTE	  1,  1,  2,  2,  5,  5, 11, 11
	.BYTE	  0,  0,128,128, 64, 64, 32, 32
	.BYTE	 22, 23, 46, 45, 90, 64,255,  0
	.BYTE	144, 16,136,  8,  4,  4,254,  0
	.BYTE 70
	.BYTE	  0, 15, 16, 47, 91,183,191,190
	.BYTE	  0,224, 16,168, 68,162, 66,130
	.BYTE	 95, 46, 21, 10,  4,  2,  1,  0
	.BYTE	 68,136, 16, 32, 64,128,  0,  0
	.BYTE 65
	.BYTE	  1,  1,  2,  2,  5, 11, 55,206
	.BYTE	  0,  0,128,128, 64,160,216,134
	.BYTE	 53, 10,  4,  2,  2,  1,  1,  0
	.BYTE	 24, 32, 64,128,128,  0,  0,  0
	.BYTE 69
	
	